local addon, data = ...

_HEADLINES = { UI={} }

local HL = _HEADLINES

local numActive = 0
local curActive = 0
local lastupdate = 0
local HLDATA = {}
local MAXMSG = 10
	
local playerName = nil
	
local math_min = math.min
local math_max = math.max
local string_format = string.format
local string_find = string.find
	
local default_settings = {
	locked=false, msgs={}, timer=5, fs=12,
	showts=true, hlsize=512,
	highlight=true, hlcnt=8,
	mmx=0, mmy=0, barx=0, bary=0, cfgx=200, cfgy=200
	}

--local default_settings = {
--	fonts = { label = 16, dist = 12 }, gpsx = 600, gpsy = 200, mmx=700, mmy=200,
--	gpsmode = GPS_MODE_ALL, hidden = {}, iconsz = 64
--}
	
local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}			
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

function HL.BuildUI()
	HL.context = UI.CreateContext(addon.identifier)
	
	HL.UI.button = UI.CreateFrame("Texture", "HL.UI.button", HL.context)
	HL.UI.button:SetTexture(addon.identifier, "img/headlines.png")
	HL.UI.button:SetLayer(1)
	HL.UI.button:SetHeight(32)
	HL.UI.button:SetWidth(32)
	
	HL.UI.hlbar = UI.CreateFrame("Text", "HL.UI.hlbar", HL.context)
	HL.UI.hlbar:SetText("Headlines")
	HL.UI.hlbar:SetFontSize(16)
	HL.UI.hlbar:SetBackgroundColor(0,0,1)
	HL.UI.hlbar:SetWidth(512)

	function HL.UI.hlbar.Event:LeftDown()
		self.MouseDown = true
		local mouseData = Inspect.Mouse()
		self.sx = mouseData.x - HL.UI.hlbar:GetLeft()
		self.sy = mouseData.y - HL.UI.hlbar:GetTop()		
	end

	function HL.UI.hlbar.Event:LeftUp()
		self.MouseDown = false
		Headline_Settings.barx = HL.UI.hlbar:GetLeft()
		Headline_Settings.bary = HL.UI.hlbar:GetTop()
	end

	function HL.UI.hlbar.Event:MouseMove()
		if self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			HL.UI.hlbar:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end
		
	local prevframe = HL.UI.hlbar
	
	HL.UI.hl = {}
	
	for x=1, 10 do
		local f = UI.CreateFrame("Text", "HL.UI.hl"..x, HL.context)
		f:SetVisible(false)
		f:SetPoint("TOPLEFT", prevframe, "BOTTOMLEFT", 0,0)
		f:SetText(string_format("HEADLINE #%d", x))
		f:SetWidth(512)
		prevframe = f
		HL.UI.hl[x] = f
	end
	
	function HL.UI.button.Event:LeftClick()
		Headline_Settings.locked = not Headline_Settings.locked
		if Headline_Settings.locked then
			HL.UI.button:SetTexture(addon.identifier, "img/headlines.png")
			HL.UI.hlbar:SetVisible(false)
		else
			HL.UI.button:SetTexture(addon.identifier, "img/headlines_bw.png")
			HL.UI.hlbar:SetVisible(true)
		end
		HL.UI.cfg:SetVisible(HL.UI.hlbar:GetVisible())
	end
	
	if MINIMAPDOCKER == nil then
		function HL.UI.button.Event:RightDown()
			if Headline_Settings.locked == false then
				self.MouseDown = true
				local mouseData = Inspect.Mouse()
				self.sx = mouseData.x - HL.UI.button:GetLeft()
				self.sy = mouseData.y - HL.UI.button:GetTop()		
			end
		end

		function HL.UI.button.Event:RightUp()
			self.MouseDown = false
			Headline_Settings.mmx = HL.UI.button:GetLeft()
			Headline_Settings.mmy = HL.UI.button:GetTop()
		end

		function HL.UI.button.Event:MouseMove()
			if self.MouseDown then
				local nx, ny
				local mouseData = Inspect.Mouse()
				nx = mouseData.x - self.sx
				ny = mouseData.y - self.sy
				HL.UI.button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
			end
		end
	end
	
	HL.UI.cfg = UI.CreateFrame("Frame", "HL.UI.cfg", HL.context)
	HL.UI.cfg:SetBackgroundColor(0,0,0)
	HL.UI.cfg:SetWidth(252)
	HL.UI.cfg:SetHeight(270)
	HL.UI.cfg:SetVisible(false)
	HL.UI.cfg:SetLayer(100)
	
	HL.UI.l_cfg = UI.CreateFrame("Text", "HL.UI.l_cfg", HL.UI.cfg)
	HL.UI.l_cfg:SetFontSize(14)
	HL.UI.l_cfg:SetText("Headlines Configuration")
	HL.UI.l_cfg:SetPoint("TOPLEFT", HL.UI.cfg, "TOPLEFT", 2,2)
	HL.UI.l_cfg:SetBackgroundColor(0,0,1)
	HL.UI.l_cfg:SetWidth(248)
	HL.UI.l_cfg:SetLayer(101)

	HL.UI.close = UI.CreateFrame("RiftButton", "HL.UI.close", HL.UI.l_cfg)
	HL.UI.close:SetSkin("close")
	HL.UI.close:SetPoint("TOPRIGHT", HL.UI.l_cfg, "TOPRIGHT", 0,0)
	HL.UI.close:SetHeight(HL.UI.l_cfg:GetHeight())
	HL.UI.close:SetWidth(HL.UI.l_cfg:GetHeight())
	HL.UI.close:SetLayer(102)
	
	function HL.UI.close.Event:LeftClick()
		HL.UI.cfg:SetVisible(false)
	end

	function HL.UI.l_cfg.Event:LeftDown()
		if Headline_Settings.locked == false then
			self.MouseDown = true
			local mouseData = Inspect.Mouse()
			self.sx = mouseData.x - HL.UI.cfg:GetLeft()
			self.sy = mouseData.y - HL.UI.cfg:GetTop()		
		end
	end

	function HL.UI.l_cfg.Event:LeftUp()
		self.MouseDown = false
		Headline_Settings.cfgx = HL.UI.cfg:GetLeft()
		Headline_Settings.cfgy = HL.UI.cfg:GetTop()
	end

	function HL.UI.l_cfg.Event:MouseMove()
		if self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			HL.UI.cfg:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end
	
	HL.UI.c_showTS = UI.CreateFrame("RiftCheckbox", "HL.UI.c_showTS", HL.UI.cfg)
	HL.UI.c_showTS:SetWidth(24)
	HL.UI.c_showTS:SetHeight(24)
	HL.UI.c_showTS:SetLayer(105)
	HL.UI.c_showTS:SetPoint("TOPLEFT", HL.UI.l_cfg, "BOTTOMLEFT", 0,2)
	
	function HL.UI.c_showTS.Event:CheckboxChange()
		if HL.UI.c_showTS:GetChecked() then
			Headline_Settings.showts = true
		else
			Headline_Settings.showts = false
		end
	end
	
	HL.UI.l_showTS = UI.CreateFrame("Text", "HL.UI.l_showTS", HL.UI.cfg)
	HL.UI.l_showTS:SetFontSize(14)
	HL.UI.l_showTS:SetLayer(105)
	HL.UI.l_showTS:SetText("Show timestamps")
	HL.UI.l_showTS:SetPoint("CENTERLEFT", HL.UI.c_showTS, "CENTERRIGHT", 2,0)
	
	HL.UI.c_highlt = UI.CreateFrame("RiftCheckbox", "HL.UI.c_highlt", HL.UI.cfg)
	HL.UI.c_highlt:SetWidth(24)
	HL.UI.c_highlt:SetHeight(24)
	HL.UI.c_highlt:SetLayer(105)
	HL.UI.c_highlt:SetPoint("TOPLEFT", HL.UI.c_showTS, "BOTTOMLEFT", 0,2)
	
	function HL.UI.c_highlt.Event:CheckboxChange()
		if HL.UI.c_highlt:GetChecked() then
			Headline_Settings.highlight = true
		else
			Headline_Settings.highlight = false
		end
	end

	HL.UI.l_highlt = UI.CreateFrame("Text", "HL.UI.l_highlt", HL.UI.cfg)
	HL.UI.l_highlt:SetFontSize(14)
	HL.UI.l_highlt:SetLayer(105)
	HL.UI.l_highlt:SetText("Highlight entries with my name")
	HL.UI.l_highlt:SetPoint("CENTERLEFT", HL.UI.c_highlt, "CENTERRIGHT", 2,0)
	
	HL.UI.l_hlcnt = UI.CreateFrame("Text", "HL.UI.l_hlcnt", HL.UI.cfg)
	HL.UI.l_hlcnt:SetPoint("TOPLEFT", HL.UI.l_highlt, "BOTTOMLEFT", 0,2)
	HL.UI.l_hlcnt:SetFontSize(14)
	HL.UI.l_hlcnt:SetLayer(105)
	HL.UI.l_hlcnt:SetText("Number of Headlines to show: ")
	
	HL.UI.s_hlcnt = UI.CreateFrame("RiftSlider", "HL.UI.s_hlcnt", HL.UI.cfg)
	HL.UI.s_hlcnt:SetLayer(105)
	HL.UI.s_hlcnt:SetWidth(200)
	HL.UI.s_hlcnt:SetRange(1,10)
	HL.UI.s_hlcnt:SetPoint("TOPLEFT", HL.UI.l_hlcnt, "BOTTOMLEFT", 0,0)
	
	function HL.UI.s_hlcnt.Event:WheelForward()
		local p = HL.UI.s_hlcnt:GetPosition()
		if p < 10 then
			p = p+1
			HL.UI.s_hlcnt:SetPosition(p)
		end
	end

	function HL.UI.s_hlcnt.Event:WheelBack()
		local p = HL.UI.s_hlcnt:GetPosition()
		if p > 1 then
			p = p-1
			HL.UI.s_hlcnt:SetPosition(p)
		end
	end
	
	function HL.UI.s_hlcnt.Event:SliderChange()
		Headline_Settings.hlcnt = HL.UI.s_hlcnt:GetPosition()
		HL.UI.l_hlcnt:SetText(string_format("Number of Headlines to show: %d", Headline_Settings.hlcnt))
	end

	HL.UI.l_fadetime = UI.CreateFrame("Text", "HL.UI.l_fadetime", HL.UI.cfg)
	HL.UI.l_fadetime:SetPoint("TOPLEFT", HL.UI.s_hlcnt, "BOTTOMLEFT", 0,-4)
	HL.UI.l_fadetime:SetFontSize(14)
	HL.UI.l_fadetime:SetLayer(105)
	HL.UI.l_fadetime:SetText("Time before fading: ")
	
	HL.UI.s_fadetime = UI.CreateFrame("RiftSlider", "HL.UI.s_fadetime", HL.UI.cfg)
	HL.UI.s_fadetime:SetLayer(105)
	HL.UI.s_fadetime:SetWidth(200)
	HL.UI.s_fadetime:SetRange(1,16)
	HL.UI.s_fadetime:SetPoint("TOPLEFT", HL.UI.l_fadetime, "BOTTOMLEFT", 0,0)
	
	function HL.UI.s_fadetime.Event:WheelForward()
		local p = HL.UI.s_fadetime:GetPosition()
		if p < 16 then
			p = p+1
			HL.UI.s_fadetime:SetPosition(p)
		end
	end

	function HL.UI.s_fadetime.Event:WheelBack()
		local p = HL.UI.s_fadetime:GetPosition()
		if p > 1 then
			p = p-1
			HL.UI.s_fadetime:SetPosition(p)
		end
	end
	
	function HL.UI.s_fadetime.Event:SliderChange()
		Headline_Settings.timer = HL.UI.s_fadetime:GetPosition()
		HL.UI.l_fadetime:SetText(string_format("Time before fading: %ds", Headline_Settings.timer))
	end

	HL.UI.l_hlsize = UI.CreateFrame("Text", "HL.UI.l_hlsize", HL.UI.cfg)
	HL.UI.l_hlsize:SetPoint("TOPLEFT", HL.UI.s_fadetime, "BOTTOMLEFT", 0,-4)
	HL.UI.l_hlsize:SetFontSize(14)
	HL.UI.l_hlsize:SetLayer(105)
	HL.UI.l_hlsize:SetText("Headline width: ")
	
	HL.UI.s_hlsize = UI.CreateFrame("RiftSlider", "HL.UI.s_hlsize", HL.UI.cfg)
	HL.UI.s_hlsize:SetLayer(105)
	HL.UI.s_hlsize:SetWidth(200)
	HL.UI.s_hlsize:SetRange(256,1024)
	HL.UI.s_hlsize:SetPoint("TOPLEFT", HL.UI.l_hlsize, "BOTTOMLEFT", 0,0)
	
	function HL.UI.s_hlsize.Event:WheelForward()
		local p = HL.UI.s_hlsize:GetPosition()
		if p < 1024 then
			p = p+1
			HL.UI.s_hlsize:SetPosition(p)
		end
	end

	function HL.UI.s_hlsize.Event:WheelBack()
		local p = HL.UI.s_hlsize:GetPosition()
		if p > 256 then
			p = p-1
			HL.UI.s_hlsize:SetPosition(p)
		end
	end
	
	function HL.UI.s_hlsize.Event:SliderChange()
		Headline_Settings.hlsize = HL.UI.s_hlsize:GetPosition()
		HL.UI.l_hlsize:SetText(string_format("Headline width: %dpx", Headline_Settings.hlsize))
		HL.UI.hlbar:SetWidth(Headline_Settings.hlsize)
		for x=1, 10 do
			HL.UI.hl[x]:SetWidth(Headline_Settings.hlsize)
		end		
	end
	
	HL.UI.l_hlfs = UI.CreateFrame("Text", "HL.UI.l_hlfs", HL.UI.cfg)
	HL.UI.l_hlfs:SetPoint("TOPLEFT", HL.UI.s_hlsize, "BOTTOMLEFT", 0,-4)
	HL.UI.l_hlfs:SetFontSize(14)
	HL.UI.l_hlfs:SetLayer(105)
	HL.UI.l_hlfs:SetText("Headline fontsize: ")
	
	HL.UI.s_hlfs = UI.CreateFrame("RiftSlider", "HL.UI.s_hlfs", HL.UI.cfg)
	HL.UI.s_hlfs:SetLayer(105)
	HL.UI.s_hlfs:SetWidth(200)
	HL.UI.s_hlfs:SetRange(6,24)
	HL.UI.s_hlfs:SetPoint("TOPLEFT", HL.UI.l_hlfs, "BOTTOMLEFT", 0,0)
	
	function HL.UI.s_hlfs.Event:WheelForward()
		local p = HL.UI.s_hlfs:GetPosition()
		if p < 24 then
			p = p+1
			HL.UI.s_hlfs:SetPosition(p)
		end
	end

	function HL.UI.s_hlfs.Event:WheelBack()
		local p = HL.UI.s_hlfs:GetPosition()
		if p > 6 then
			p = p-1
			HL.UI.s_hlfs:SetPosition(p)
		end
	end
	
	function HL.UI.s_hlfs.Event:SliderChange()
		Headline_Settings.fs = HL.UI.s_hlfs:GetPosition()
		HL.UI.l_hlfs:SetText(string_format("Headline fontsize: %d", Headline_Settings.fs))
		for x=1, 10 do
			HL.UI.hl[x]:SetFontSize(Headline_Settings.fs)
		end		
	end

	HL.UI.test = UI.CreateFrame("RiftButton", "HL.UI.test", HL.UI.cfg)
	HL.UI.test:SetSkin("default")
	HL.UI.test:SetPoint("BOTTOMCENTER", HL.UI.cfg, "BOTTOMCENTER", 0,-4)
	HL.UI.test:SetText("Test!")
	HL.UI.test:SetLayer(102)
	
	function HL.UI.test.Event:LeftClick()
		HL.ShowTestData()
	end
	
end

function HL.Event_Addon_SavedVariables_Load_End(a)
	if a == addon.identifier then
		if Headline_Settings == nil then
			local sw = math.floor(UIParent:GetWidth()/2)-16
			local sh = math.floor(UIParent:GetHeight()/2)-16
			Headline_Settings = {mmx=sw, mmy=sh, barx=(sw+34), bary=sh}
		end
		MergeTable(Headline_Settings, default_settings)
		if MINIMAPDOCKER then
			MINIMAPDOCKER.Register(addon.identifier, HL.UI.button)
		else
			HL.UI.button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Headline_Settings.mmx, Headline_Settings.mmy)
		end
		HL.UI.hlbar:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Headline_Settings.barx, Headline_Settings.bary)
		HL.UI.cfg:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Headline_Settings.cfgx, Headline_Settings.cfgy)
		if Headline_Settings.locked then
			HL.UI.button:SetTexture(addon.identifier, "img/headlines.png")
			HL.UI.hlbar:SetVisible(false)
			HL.UI.cfg:SetVisible(false)
		else
			HL.UI.button:SetTexture(addon.identifier, "img/headlines_bw.png")
			HL.UI.hlbar:SetVisible(true)
			HL.UI.cfg:SetVisible(true)
		end	
		HL.UI.c_showTS:SetChecked(Headline_Settings.showts)
		HL.UI.c_highlt:SetChecked(Headline_Settings.highlight)
		if Headline_Settings.timer == 1 then
			HL.UI.s_fadetime:SetPosition(16)
		end
		HL.UI.s_fadetime:SetPosition(Headline_Settings.timer)
		if Headline_Settings.hlcnt == 1 then
			HL.UI.s_hlcnt:SetPosition(10)
		end
		HL.UI.s_hlcnt:SetPosition(Headline_Settings.hlcnt)
		if Headline_Settings.hlsize == 256 then
			HL.UI.s_hlsize:SetPosition(1024)
		end
		HL.UI.s_hlsize:SetPosition(Headline_Settings.hlsize)
		if Headline_Settings.fs == 6 then
			HL.UI.s_hlfs:SetPosition(24)
		end
		HL.UI.s_hlfs:SetPosition(Headline_Settings.fs)

		table.insert(Event.System.Update.Begin, { HL.Event_System_Update_Begin, addon.identifier, "Event.System.Update.Begin" })
	end
end

local LONGSTRING = "1234567890-=qwertyuiop[]asdfghjkl;'#zxcvbnm,./!\"$%^&*()_+QWERTYUIOP{}ASDFGHJKL:@~ZXCVBNM<>?"

function HL.ShowTestData()
	local ct = Inspect.Time.Real()
	for x=10, 1, -1 do
		if playerName and (x==1 or x==4 or x==7 or x==10) then
			table.insert(HLDATA, 1, {t=ct+x, ts=os.date("%H:%M:%S", os.time()+x), p=true, m=string_format("TEST MESSAGE >%s< #%d [%s] (%s)(%s)(%s)",playerName, x,os.date("%H:%M:%S", os.time()+x),LONGSTRING,LONGSTRING,LONGSTRING )})
		else
			table.insert(HLDATA, 1, {t=ct+x, ts=os.date("%H:%M:%S", os.time()+x), p=false, m=string_format("TEST MESSAGE #%d [%s] (%s)(%s)(%s)",x,os.date("%H:%M:%S", os.time()+x),LONGSTRING,LONGSTRING,LONGSTRING )})
		end
		table.remove(HLDATA, MAXMSG+1)
	end
end

function HL.Event_System_Update_Begin()
	if playerName == nil then
		local d = Inspect.Unit.Detail("player")
		if d then
			playerName = d.name
		end
	end
	
	if #HLDATA > 0 then
		curActive = 0
		local ctx = Inspect.Time.Real()
		if ctx - lastupdate > 0.1 then
			table.sort(HLDATA, function(a,b) return a.t > b.t end)
			local toffset = ctx - Headline_Settings.timer
			for x=math_min(#HLDATA, MAXMSG, Headline_Settings.hlcnt), 1, -1 do
				local td = HLDATA[x].t - toffset
				if td > -2 then
					curActive = curActive+1
					if Headline_Settings.showts then
						HL.UI.hl[x]:SetText(string_format("[%s] %s", HLDATA[x].ts, HLDATA[x].m))
					else
						HL.UI.hl[x]:SetText(string_format("%s", HLDATA[x].m))
					end
					HL.UI.hl[x]:SetVisible(true)
					if td < 0 then						
						local al = 1-(td/-2)
						HL.UI.hl[x]:SetAlpha(al)
					else
						HL.UI.hl[x]:SetAlpha(1)
					end
					if HLDATA[x].p and Headline_Settings.highlight then
						HL.UI.hl[x]:SetFontColor(0,1,0)
					else
						HL.UI.hl[x]:SetFontColor(1,1,1)
					end
				else
					HL.UI.hl[x]:SetVisible(false)
				end
			end

			if curActive ~= numActive then
				for x=curActive+1, MAXMSG do
					HL.UI.hl[x]:SetVisible(false)
				end
				numActive = curActive
			end
			lastupdate = ctx
		end
	end
end

-- /script _HEADLINES.Event_Chat_Notify({message="Stillmoor has been Conquered!"})
-- /script _HEADLINES.Event_Chat_Notify({message="Conquest Power has been awarded for control of Extractors.  Next award in 10 minutes."})

function HL.Event_Chat_Notify(i)
	table.insert(Headline_Settings.msgs, 1, {t=Inspect.Time.Server(), m=i.message})
	if playerName and string_find(i.message, playerName) then
		table.insert(HLDATA, 1, {t=Inspect.Time.Real(), p=true, ts=os.date("%H:%M:%S"), m=i.message})
	else
		table.insert(HLDATA, 1, {t=Inspect.Time.Real(), p=false, ts=os.date("%H:%M:%S"), m=i.message})
	end
	table.remove(HLDATA, MAXMSG+1)
	print("MSG: "..i.message)
end

HL.BuildUI()

table.insert(Event.Addon.SavedVariables.Load.End, { HL.Event_Addon_SavedVariables_Load_End, addon.identifier, "Event.Addon.SavedVariables.Load.End" })
table.insert(Event.Chat.Notify, { HL.Event_Chat_Notify, addon.identifier, "Event.Chat.Notify"})
